#!/bin/perl

#
#  Screentone Pattern Generator(gen_tone.pl)   ver0.01    2021 TANE
#

use strict;
use Getopt::Long qw(:config posix_default no_ignore_case gnu_compat);

my $patwidth=0.6 ;
my $density =100 ;
my $genmode ="dot" ;
my $help=0;

$,=' ' ;
my @args=@ARGV ;
for( my $i=0;$i<=$#args;$i++ ){
    $args[$i] =~ s/^--/\\-\\-/ ;
}

if( !GetOptions(
        "mode|m=s"    => \$genmode,
        "width|w=s"   => \$patwidth,
        "density|d=s" => \$density,
        "help|h"      => \$help
    )){
    exit(1) ;
}

if( $help!=0 ){
    &printHelp() ;
    exit(0) ;
}

if( ($genmode ne "dot"     ) &&
    ($genmode ne "sqr"     ) &&
    ($genmode ne "rhombus" ) &&
    ($genmode ne "linerr"  ) &&
    ($genmode ne "linedr"  ) &&
    ($genmode ne "lineh"   ) &&
    ($genmode ne "linev"   ) ){
    printf(STDERR "Error : Unknown generate mode %s .\n",$genmode) ;
    exit(1) ;
}


&print_header() ;

if(     $genmode eq "dot" ){
    &generate_dotpattern($density, $patwidth) ;
}elsif( $genmode eq "sqr" ){
    &generate_sqrpattern($density, $patwidth, 0.0) ;
}elsif( $genmode eq "rhombus" ){
    &generate_sqrpattern($density, $patwidth, 45.0) ;
}elsif( $genmode eq "linerr" ){
    &generate_linepattern($density, $patwidth, "RisingToTheRight") ;
}elsif( $genmode eq "linedr" ){
    &generate_linepattern($density, $patwidth, "DownwardToTheRight") ;
}elsif( $genmode eq "lineh" ){
    &generate_linepattern($density, $patwidth, "Horizontal") ;
}elsif( $genmode eq "linev" ){
    &generate_linepattern($density, $patwidth, "Vertical") ;
}

&print_footer() ;

exit(0) ;


sub generate_dotpattern
{
    my ($density, $patwidth)=@_ ;

    $density=int($density/2.0)*2;
    my $texsize=135.47 ;
    my $pitch  =$texsize/$density ;

    my $linenum=0 ;
    for( my $y=0 ; $y<=$density ; $y+=1 ){
        my $offset=($linenum % 2)==0 ? 0 : $pitch ;
        for( my $x=0 ; $x<=$density/2 ; $x+=1 ){
            print '  <circle' . "\n" .
                '       style="fill:#000000;fill-opacity:1;stroke:none;stroke-width:0.1;stroke-linecap:round;stroke-miterlimit:4;stroke-dasharray:none;paint-order:stroke markers fill"' . "\n" ;
            printf("       cx=\"%f\" cy=\"%f\" r=\"%f\"  />\n", $x * $pitch * 2.0 + $offset,  $y * $pitch, $patwidth) ;
        }
        $linenum++ ;
    }
}

sub generate_sqrpattern
{
    my ($density, $patwidth, $rotate)=@_ ;

    $density=int($density/2.0)*2;
    my $texsize=135.47 ;
    my $pitch  =$texsize/$density ;

    my $linenum=0 ;
    for( my $y=0 ; $y<=$density ; $y+=1 ){
        my $offset=($linenum % 2)==0 ? 0 : $pitch ;
        for( my $x=0 ; $x<=$density/2 ; $x+=1 ){
            print '  <rect' . "\n" .
                '       style="fill:#000000;fill-opacity:1;stroke-linecap:round;paint-order:stroke markers fill;"' . "\n" ;
            if( $rotate==0.0 ){
                printf("       x=\"%f\" y=\"%f\" width=\"%f\" height=\"%f\" />\n",
                       $x * $pitch * 2.0 - $patwidth + $offset,  $y * $pitch - $patwidth, $patwidth*2.0, $patwidth*2.0) ;
            }else{
                printf("       width=\"%f\" height=\"%f\" x=\"%f\" y=\"%f\" transform=\"translate(%f,%f) rotate(%f,%f,%f) \" />\n",
                       $patwidth*2.0, $patwidth*2.0,
                       0.0, 0.0,
                       $x * $pitch * 2.0 - $patwidth + $offset,  $y * $pitch - $patwidth,
                       $rotate, $patwidth, $patwidth );
            }
        }
        $linenum++ ;
    }
}

sub generate_linepattern
{
    my ($density, $patwidth, $mode)=@_ ;

    $density=int($density/2.0)*2;
    my $texsize=135.47 ;
    my $pitch  =$texsize/$density ;

    if( $mode eq "RisingToTheRight" ){
        for( my $x=0 ; $x<=$density ; $x+=1 ){
            print '  <path' . "\n" ;
            print '       style="fill:none;stroke:#000000;stroke-width:' .
                          sprintf("%f",$patwidth) .
                          'px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"' . "\n" ;
            printf("       d=\"M %f,%f %f,%f\" />\n", $x * $pitch, -$pitch, -$pitch, $x * $pitch) ;

            print '  <path' . "\n" ;
            print '       style="fill:none;stroke:#000000;stroke-width:' .
                          sprintf("%f",$patwidth) .
                          'px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"' . "\n" ;
            printf("       d=\"M %f,%f %f,%f\" />\n", $x * $pitch, $pitch*$density+$pitch, $pitch*$density+$pitch, $x * $pitch) ;
        }

        print '  <path' . "\n" ;
        print '       style="fill:none;stroke:#000000;stroke-width:' .
                      sprintf("%f",$patwidth) .
                      'px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"' . "\n" ;
        printf("       d=\"M %f,%f %f,%f\" />\n", ($density+1) * $pitch, -$pitch, -$pitch, ($density+1) * $pitch) ;

    }elsif( $mode eq "DownwardToTheRight" ){
        for( my $x=0 ; $x<=$density ; $x+=1 ){
            print '  <path' . "\n" ;
            print '       style="fill:none;stroke:#000000;stroke-width:' .
                          sprintf("%f",$patwidth) .
                          'px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"' . "\n" ;
            printf("       d=\"M %f,%f %f,%f\" />\n", $x * $pitch, -$pitch, $pitch*$density+$pitch, $pitch*$density - $x * $pitch) ;

            print '  <path' . "\n" ;
            print '       style="fill:none;stroke:#000000;stroke-width:' .
                          sprintf("%f",$patwidth) .
                          'px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"' . "\n" ;
            printf("       d=\"M %f,%f %f,%f\" />\n", $pitch*$density - $x * $pitch, $pitch*$density+$pitch, -$pitch, $x * $pitch) ;
        }

        print '  <path' . "\n" ;
        print '       style="fill:none;stroke:#000000;stroke-width:' .
                      sprintf("%f",$patwidth) .
                      'px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"' . "\n" ;
        printf("       d=\"M %f,%f %f,%f\" />\n", -$pitch, -$pitch, ($density+1) * $pitch, ($density+1) * $pitch) ;

    }elsif( $mode eq "Horizontal" ){
        for( my $y=0 ; $y<=$density ; $y+=1 ){
            print '  <path' . "\n" ;
            print '       style="fill:none;stroke:#000000;stroke-width:' .
                          sprintf("%f",$patwidth) .
                          'px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"' . "\n" ;
            printf("       d=\"M %f,%f %f,%f\" />\n", -$pitch, $y * $pitch + $pitch/2.0, $pitch*$density+$pitch, $y * $pitch + $pitch/2.0) ;

        }
    }elsif( $mode eq "Vertical" ){
        for( my $x=0 ; $x<=$density ; $x+=1 ){
            print '  <path' . "\n" ;
            print '       style="fill:none;stroke:#000000;stroke-width:' .
                          sprintf("%f",$patwidth) .
                          'px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"' . "\n" ;
            printf("       d=\"M %f,%f %f,%f\" />\n", $x * $pitch + $pitch/2.0, -$pitch, $x * $pitch + $pitch/2.0, $pitch*$density+$pitch) ;

        }
    }
}

sub print_header
{
    print '<?xml version="1.0" encoding="UTF-8" standalone="no"?>' . "\n" .
          '<!-- Base SVG created with Inkscape (http://www.inkscape.org/) -->' . "\n" .
          '<!-- Create by "gen_tone.pl' , @args , '" -->' . "\n" .
          '' . "\n" .
          '<svg' . "\n" .
          '   width="512"' . "\n" .
          '   height="512"' . "\n" .
          '   viewBox="0 0 135.46667 135.46667"' . "\n" .
          '   version="1.1"' . "\n" .
          '   id="svg5"' . "\n" .
          '   inkscape:version="1.1 (c68e22c387, 2021-05-23)"' . "\n" .
          '   sodipodi:docname="tone_test.svg"' . "\n" .
          '   xmlns:inkscape="http://www.inkscape.org/namespaces/inkscape"' . "\n" .
          '   xmlns:sodipodi="http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"' . "\n" .
          '   xmlns="http://www.w3.org/2000/svg"' . "\n" .
          '   xmlns:svg="http://www.w3.org/2000/svg">' . "\n" .
          '  <sodipodi:namedview' . "\n" .
          '     id="namedview7"' . "\n" .
          '     pagecolor="#ffffff"' . "\n" .
          '     bordercolor="#666666"' . "\n" .
          '     borderopacity="1.0"' . "\n" .
          '     inkscape:pageshadow="2"' . "\n" .
          '     inkscape:pageopacity="0.0"' . "\n" .
          '     inkscape:pagecheckerboard="0"' . "\n" .
          '     inkscape:document-units="mm"' . "\n" .
          '     showgrid="true"' . "\n" .
          '     units="px"' . "\n" .
          '     inkscape:current-layer="layer1">' . "\n" .
          '    <inkscape:grid' . "\n" .
          '       type="xygrid"' . "\n" .
          '       id="grid824" />' . "\n" .
          '  </sodipodi:namedview>' . "\n" .
          '  <defs' . "\n" .
          '     id="defs2" />' . "\n" .
          '  <g' . "\n" .
          '     inkscape:label="Layer 1"' . "\n" .
          '     inkscape:groupmode="layer"' . "\n" .
          '     id="layer1">' . "\n" ;
}

sub print_footer
{
    print '  </g>' . "\n" .
          '</svg>' . "\n" ;
}

sub printHelp {
    print(
        "Screentone Pattern Generator   Ver0.01   2021 TANE\n" .
        "\n" .
        "Usage: $0 [OPTIONs] > foo.svg\n" .
        "  -m str  or --mode=str     : Specify pattern. (default=dot)\n" .
        "                                dot     : circle dot pattern.\n" .
        "                                sqr     : square dot pattern.\n" .
        "                                rhombus : rhombus dot pattern.\n" .
        "                                linerr  : riseward to the right line pattern.\n" .
        "                                linedr  : downward to the right line pattern.\n" .
        "                                lineh   : horizontal line pattern.\n" .
        "                                linev   : vertical line pattern.\n" .
        "  -w num  or --patwidth=num : Specify dot/line size.    (default=0.6)\n" .
        "  -d num  or --density=num  : Specify dot/line density. (default=100)\n" .
        "  -h      or --help         : Print this help.\n" .
        "\n" .
        "The generated screentone SVG file must be converted to a grayscale BMP file for use with SAI2.\n" .
        "The following is an example of conversion using convert command of ImageMagick.\n" .
        "   1. $0 -m dot -d  80 -w 0.4 > foo.svg\n" .
        "   2. convert foo.svg  -colorspace Gray -type Grayscale -depth 8 foo.png\n" .
        "   3. convert foo.png  -define bmp:format=bmp3 -compress None foo.bmp\n"
        );
}
